/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.Statistics");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.CountersClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.DataTableContainer");
//dojo.require("dijit.layout.TabContainer");
//dojo.require("dijit.layout.ContentPane");

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var CountersClient = com.ibm.team.repository.web.client.internal.CountersClient;
var DataTableContainer = com.ibm.team.repository.web.ui.internal.admin.DataTableContainer;

var TIME_STATS = [
	{label1: "Web Services", label2: "Longest Average Time", groupName: "web service", facetName: "1. elapsed time (seconds)", sortKey: "average", unit: "Seconds"},
	{label1: "Asynchronous Tasks", label2: "Longest Average Time", groupName: "asynchronous tasks", facetName: "elapsed time (seconds)", sortKey: "average", unit: "Seconds"},
	{label1: "Web Services", label2: "Top Count", groupName: "web service", facetName: "1. elapsed time (seconds)", sortKey: "count", unit: "Count"},
	{label1: "Asynchronous Tasks", label2: "Top Count", groupName: "asynchronous tasks", facetName: "elapsed time (seconds)", sortKey: "count", unit: "Count"}
];

var MEMORY_STATS = [
	{label1: "Current Cache", label2: "Top Total Size", groupName: "Current State Cache Details", facetName: "estimated size", sortKey: "total", unit: "Bytes"},
	{label1: "Current Cache", label2: "Top Hit Count", groupName: "Current State Cache Details", facetName: "hits", sortKey: "count", unit: "Count"},
	{label1: "Current Cache", label2: "Top Miss Count", groupName: "Current State Cache Details", facetName: "misses", sortKey: "count", unit: "Count"},
	{label1: "State Cache", label2: "Top Total Size", groupName: "State Cache Details", facetName: "estimated size", sortKey: "total", unit: "Bytes"},
	{label1: "State Cache", label2: "Top Hit Count", groupName: "State Cache Details", facetName: "hits", sortKey: "count", unit: "Count"},
	{label1: "State Cache", label2: "Top Miss Count", groupName: "State Cache Details", facetName: "misses", sortKey: "count", unit: "Count"}
];

function getSortValueForFacetAndKey(counter, facetName, sortKey) {
	for (var i = 0; i < counter.facets.length; i++) {
		if (counter.facets[i].name === facetName) {
			return counter.facets[i][sortKey];
		}
	}
	return null;
}

function getFacetForName(counter, facetName) {
	for (var i = 0; i < counter.facets.length; i++) {
		if (counter.facets[i].name === facetName) {
			return counter.facets[i];
		}
	}
	return null;
}

function getCounterGroupForName(groups, groupName){
	for (var i = 0; i < groups.length; i++) {
		if (groups[i].name === groupName) {
			return groups[i];
		}
	}
	return null;
}

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.Statistics", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/Statistics.html"),

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.Statistics.superclass.postCreate.apply(this);
		this._fetchAndDisplayStatisticsInfo();
	},
	
	_fetchAndDisplayStatisticsInfo: function() {
		var srh= new ServiceResponseHandler(this, "_displayStatisticsInfo", "_handleStatisticsFetchError");
		CountersClient.getAllCounters(srh);
	},

	_displayStatisticsInfo: function(data) {
		this._clearAllContent();
		var groups = data.groups;
		if (groups && groups.length > 0) {
			var timeTab = this._buildStatisticsTab(groups, TIME_STATS);
			this._timeStatistics.appendChild(timeTab);
			var memoryTab = this._buildStatisticsTab(groups, MEMORY_STATS);
			this._memoryStatistics.appendChild(memoryTab);
		} else{
			//TODO display msg
			var noResults = document.createElement("div");
			noResults.innerHTML("NO RESULTS");
			this._timeStatistics.appendChild(noResults);
		}
	},

	_clearAllContent: function() {
		//TODO update
		dojox.data.dom.removeChildren(this._timeStatistics);
		dojox.data.dom.removeChildren(this._memoryStatistics);
	},
		
	_buildStatisticsTab: function(groups, statTarget) {
		var timeDiv = document.createElement("div");
		var self = this;
		var statsEmpty = true;
		dojo.forEach(statTarget, function(stats){
			var selectedGroup = getCounterGroupForName(groups, stats.groupName);
			if (selectedGroup) {
				var selectedCounters = selectedGroup.counters;
				var statList = self._fetchSortedBlockByFacetKey(selectedCounters, stats.facetName, stats.sortKey, 10);
				if (statList && statList.length > 0) {
					self._appendStatsBlock(timeDiv, stats, statList);
					statsEmpty = false;
				}				
			} 
			if (statsEmpty) {
				self._appendNoResultsBlock(timeDiv, stats);
			}
		});
		return timeDiv;	
	},
	
	_appendStatsBlock: function(node, stats, dataList){
		var statBlock = document.createElement("div");
		statBlock.className = "stats-block";
		
		var args = {
			heading: {value: stats.label2, label: stats.label1},
			headers: [stats.unit, "Name"]
		};
		var statTable = new DataTableContainer(args);
		
		var self = this;
		dojo.forEach(dataList, function(item){
			var formattedValue = self._getFormattedSortValue(item, stats.facetName, stats.sortKey);
			if (formattedValue) {
				var rowData = {
					0: {
						text: formattedValue,
						style: "numeric rank tight"
					},
					1: {
						text: item.name
					}
				};
				statTable.addRow(rowData, 2);
			}
		});
		node.appendChild(statTable.domNode);
	},
	
	_appendNoResultsBlock: function(node, stats){
		var statBlock = document.createElement("div");
		statBlock.className = "stats-block";
		
		var args = {
			heading: {value: stats.label2, label: stats.label1},
			headers: ["No Results for Selected Counter Group"]
		};
		var statTable = new DataTableContainer(args);
		node.appendChild(statTable.domNode);
	},
		
	_getFormattedSortValue: function(counter, facetName, sortKey) {
		var value = getSortValueForFacetAndKey(counter, facetName, sortKey);
		var facet = getFacetForName(counter, facetName);
		if (!value || !facet){
			return null;
		}
		return this._formatStatistic(value, facet, sortKey);
	},
	
	_formatStatistic: function(statValue, facet, sortKey){
		var flags = {places:4};
		if (facet.type === "SCALAR" || sortKey === "count" || sortKey === "total") {
			flags = {places:0};
		}
		return dojo.number.format(statValue, flags);
	},
	
	_fetchSortedBlockByFacetKey: function(counters, facetName, sortKey, count){
		counters.sort(
			function(a, b) {
				return getSortValueForFacetAndKey(a, facetName, sortKey) < getSortValueForFacetAndKey(b, facetName, sortKey) ? 1 : -1;
			});
		return counters.slice(0, count);
	},
	
	_handleStatisticsFetchError: function(errorObj) {
		this._displayErrorMessage("Error fetching server statistics", errorObj);
	},	

	_displayErrorMessage: function(message, errorObj) {
		if (errorObj) {
			message += " (detailed message: " + 
				(errorObj.status === 0 ? "server unreachable - check network connectivity or contact server admin" : errorObj.message) + ").";
		}
		this._displayMessage(message, "errorMsg");
	},
	
	// do not call directly
	_displayMessage: function(message, className) {
		dojox.data.dom.removeChildren(this.messageArea);
		this.messageArea.className= className;
		this.messageArea.appendChild(document.createTextNode(message));
	}

});

})();